// Author: A.Voss@FH-Aachen.de
//
// use a mutex as start signal
// tags: thread

#include <iostream>
#include <thread>
#include <condition_variable>
#include <chrono>
#include <string>
#include <vector>
#include <algorithm>
using namespace std;

#include "../../include/tools.h"

// thread-function
void Work(mutex & signal, int & cnt) 
{
    tools_log(); cout << "thread started, wait for signal" << endl;
    
    cnt = 0;
    // try to lock, if that fails, do work (inc cnt)
    while (!signal.try_lock())
    {
        ++cnt;
    }
    // ok, got mutex, remember to unlock
    signal.unlock();

    tools_log(); cout << "thread works" << endl;     

    this_thread::sleep_for(chrono::seconds(1));

    tools_log(); cout << "thread done" << endl;     
}

int main()
{
    mutex signal;

    tools_log(); cout << "start threads t1 and t2 and count" << endl;

    signal.lock();

    int cnt1,cnt2;
    
    thread t1(Work,ref(signal),ref(cnt1)); tools_registerthreadname(t1); 
    thread t2(Work,ref(signal),ref(cnt2)); tools_registerthreadname(t2); 

    tools_log(); cout << "do work in main thread" << endl;
    
    this_thread::sleep_for(chrono::seconds(1));

    tools_log(); cout << "work done, signal threads and wait" << endl;
    
    // reset signal, now try_lock works
    signal.unlock();

    t1.join();
    t2.join();

    tools_log(); cout << "joined, cnt1=" << cnt1 << ", cnt2=" << cnt2 << endl;
    
    return EXIT_SUCCESS;
}
