// Author: A.Voss@FH-Aachen.de
//
// program shows conditional_variable pattern
// tags: thread, mutex, condition_variable

#include <iostream>
#include <thread>
#include <condition_variable>
#include <chrono>
#include <string>
using namespace std;

#include "../../include/tools.h"

// thread-function
void Work(int version, condition_variable & cv, mutex & m, bool & workdone) 
{
    tools_log(); cout << "thread started" << endl;

    std::unique_lock<std::mutex> lk(m);
    tools_log(); cout << "got mutex, now wait for signal" << endl;

    switch(version) {
        case 1:
        {
            cv.wait(lk);
        } break;
        case 2:
        {
            // also prevents wait from so-called spuriously unlocks
            while(!workdone)
                cv.wait(lk);
        } break;
        case 3:
        {
            // short form, identical to case 2
            cv.wait(lk,[=]{return workdone;});
        } break;
    }
    tools_log(); cout << "got signal, do work for 2 sec" << endl;
    
    this_thread::sleep_for(std::chrono::seconds(2));
    
    tools_log(); cout << "work and thread done" << endl;     
}

int main()
{
    tools_log(); cout << "start thread t" << endl;

    condition_variable cv;
    mutex m;
    bool workdone = false;

    int version = 2;    // 1..3
    
    thread t(Work,version,ref(cv),ref(m),ref(workdone));tools_registerthreadname(t);

    // comment out sleep and version 1 will hang
    //this_thread::sleep_for(std::chrono::milliseconds(10));
    
    {
    std::unique_lock<std::mutex> lk(mutex);
    tools_log(); cout << "do main work" << endl;
    workdone = true;
    cv.notify_one();
    }

    tools_log(); cout << "and wait for join" << endl;

    // wait for thread
    t.join();

    tools_log(); cout << "joined, done" << endl;
    
    return EXIT_SUCCESS;
}
